/*
 * Copyright (c) 2010-2014 Isode Limited.
 * All rights reserved.
 * See the COPYING file for more information.
 */

#include <Swiften/Serializer/StanzaSerializer.h>

#include <sstream>
#include <typeinfo>
#include <iostream>

#include <Swiften/Base/foreach.h>
#include <Swiften/Serializer/XML/XMLElement.h>
#include <Swiften/Serializer/XML/XMLRawTextNode.h>
#include <Swiften/Serializer/PayloadSerializer.h>
#include <Swiften/Serializer/PayloadSerializerCollection.h>
#include <Swiften/Elements/Stanza.h>

namespace Swift {

StanzaSerializer::StanzaSerializer(const std::string& tag, PayloadSerializerCollection* payloadSerializers, const boost::optional<std::string>& explicitNS) : tag_(tag), payloadSerializers_(payloadSerializers), explicitDefaultNS_(explicitNS) {
}

SafeByteArray StanzaSerializer::serialize(boost::shared_ptr<ToplevelElement> element) const {
	if (explicitDefaultNS_) {
		return serialize(element, explicitDefaultNS_.get());
	}
	else {
		return serialize(element, "");
	}
}

SafeByteArray StanzaSerializer::serialize(boost::shared_ptr<ToplevelElement> element, const std::string& xmlns) const {
	boost::shared_ptr<Stanza> stanza(boost::dynamic_pointer_cast<Stanza>(element));

	XMLElement stanzaElement(tag_, explicitDefaultNS_ ? explicitDefaultNS_.get() : xmlns);
	if (stanza->getFrom().isValid()) {
		stanzaElement.setAttribute("from", stanza->getFrom());
	}
	if (stanza->getTo().isValid()) {
		stanzaElement.setAttribute("to", stanza->getTo());
	}
	if (!stanza->getID().empty()) {
		stanzaElement.setAttribute("id", stanza->getID());
	}
	setStanzaSpecificAttributes(stanza, stanzaElement);

	std::string serializedPayloads;
	foreach (const boost::shared_ptr<Payload>& payload, stanza->getPayloads()) {
		PayloadSerializer* serializer = payloadSerializers_->getPayloadSerializer(payload);
		if (serializer) {
			serializedPayloads += serializer->serialize(payload);
		}
		else {
			std::cerr << "Could not find serializer for " << typeid(*(payload.get())).name() << std::endl;
		}
	}
	if (!serializedPayloads.empty()) {
		stanzaElement.addNode(boost::shared_ptr<XMLNode>(new XMLRawTextNode(serializedPayloads)));
	}

	return createSafeByteArray(stanzaElement.serialize());
}

}
